from django.test import TestCase
from django.core.exceptions import PermissionDenied
from django.contrib.auth.models import AnonymousUser

from nose.tools import raises

from geokey.projects.tests.model_factories import UserF, ProjectF
from geokey.projects.models import Project
from .model_factories import GroupingFactory

from ..models import Grouping


class TestPublicViews(TestCase):
    def setUp(self):
        self.admin = UserF.create()
        self.view1_user = UserF.create()
        self.view2_user = UserF.create()
        self.project = ProjectF(
            add_admins=[self.admin],
            **{'isprivate': False}
        )
        self.view1 = GroupingFactory(add_viewers=[self.view1_user], **{
            'project': self.project
        })
        self.view2 = GroupingFactory(add_viewers=[self.view2_user], **{
            'project': self.project
        })
        self.view1_public = GroupingFactory(add_viewers=[self.view1_user], **{
            'project': self.project,
            'isprivate': False
        })
        self.view2_public = GroupingFactory(add_viewers=[self.view2_user], **{
            'project': self.project,
            'isprivate': False
        })

    def test_get_list_with_admin(self):
        views = Grouping.objects.get_list(self.admin, self.project.id)
        self.assertEqual(len(views), 4)

    def test_get_list_with_anonymous(self):
        views = Grouping.objects.get_list(AnonymousUser(), self.project.id)
        self.assertEqual(len(views), 2)
        self.assertIn(self.view1_public, views)
        self.assertIn(self.view2_public, views)
        self.assertNotIn(self.view1, views)
        self.assertNotIn(self.view2, views)

    def test_get_list_with_non_member(self):
        non_member = UserF.create()
        views = Grouping.objects.get_list(non_member, self.project.id)
        self.assertEqual(len(views), 2)
        self.assertIn(self.view1_public, views)
        self.assertIn(self.view2_public, views)
        self.assertNotIn(self.view1, views)
        self.assertNotIn(self.view2, views)

    def test_get_list_with_view1_user(self):
        views = Grouping.objects.get_list(self.view1_user, self.project.id)
        self.assertEqual(len(views), 3)
        self.assertIn(self.view1_public, views)
        self.assertIn(self.view2_public, views)
        self.assertIn(self.view1, views)
        self.assertNotIn(self.view2, views)

    def test_get_list_with_view2_user(self):
        views = Grouping.objects.get_list(self.view2_user, self.project.id)
        self.assertEqual(len(views), 3)
        self.assertIn(self.view1_public, views)
        self.assertIn(self.view2_public, views)
        self.assertNotIn(self.view1, views)
        self.assertIn(self.view2, views)


class TestSinglePublicView(TestCase):
    def setUp(self):
        self.admin = UserF.create()
        self.view_user = UserF.create()
        self.project = ProjectF(
            add_admins=[self.admin],
            **{'isprivate': False}
        )
        self.private_view = GroupingFactory(add_viewers=[self.view_user], **{
            'project': self.project
        })
        self.public_view = GroupingFactory(add_viewers=[self.view_user], **{
            'project': self.project,
            'isprivate': False
        })

    def test_get_public_view_with_admin(self):
        view = Grouping.objects.get_single(
            self.admin, self.project.id, self.public_view.id)
        self.assertEqual(view, self.public_view)

    def test_get_private_view_with_admin(self):
        view = Grouping.objects.get_single(
            self.admin, self.project.id, self.private_view.id)
        self.assertEqual(view, self.private_view)

    def test_get_public_view_with_view_user(self):
        view = Grouping.objects.get_single(
            self.view_user, self.project.id, self.public_view.id)
        self.assertEqual(view, self.public_view)

    def test_get_private_view_with_view_user(self):
        view = Grouping.objects.get_single(
            self.view_user, self.project.id, self.private_view.id)
        self.assertEqual(view, self.private_view)

    def test_get_public_view_with_anonymous(self):
        view = Grouping.objects.get_single(
            AnonymousUser(), self.project.id, self.public_view.id)
        self.assertEqual(view, self.public_view)

    @raises(Grouping.DoesNotExist)
    def test_get_private_view_with_anonymous(self):
        Grouping.objects.get_single(
            AnonymousUser(), self.project.id, self.private_view.id)

    def test_get_public_view_with_some_dude(self):
        view = Grouping.objects.get_single(
            UserF.create(), self.project.id, self.public_view.id)
        self.assertEqual(view, self.public_view)

    @raises(Grouping.DoesNotExist)
    def test_get_private_view_with_some_dude(self):
        Grouping.objects.get_single(
            UserF.create(), self.project.id, self.private_view.id)


class TestManager(TestCase):
    def setUp(self):
        self.admin = UserF.create()
        self.contributor = UserF.create()
        self.view1_user = UserF.create()
        self.view2_user = UserF.create()
        self.non_member = UserF.create()
        self.project = ProjectF(
            add_admins=[self.admin],
            add_contributors=[self.contributor]
        )
        self.view1 = GroupingFactory(add_viewers=[self.view1_user], **{
            'project': self.project
        })
        self.view2 = GroupingFactory(add_viewers=[self.view2_user], **{
            'project': self.project
        })

    def test_get_views_with_admin(self):
        views = Grouping.objects.get_list(self.admin, self.project.id)
        self.assertEqual(len(views), 2)

    def test_get_views_with_view1_user(self):
        views = Grouping.objects.get_list(self.view1_user, self.project.id)
        self.assertEqual(len(views), 1)
        self.assertIn(self.view1, views)
        self.assertNotIn(self.view2, views)

    def test_get_views_with_view2_user(self):
        views = Grouping.objects.get_list(self.view2_user, self.project.id)
        self.assertEqual(len(views), 1)
        self.assertIn(self.view2, views)
        self.assertNotIn(self.view1, views)

    def test_get_views_with_contributor(self):
        views = Grouping.objects.get_list(self.contributor, self.project.id)
        self.assertEqual(len(views), 0)

    @raises(Project.DoesNotExist)
    def test_get_views_with_non_member(self):
        Grouping.objects.get_list(self.non_member, self.project.id)

    def test_get_single_view_with_admin(self):
        view = Grouping.objects.get_single(
            self.admin, self.project.id, self.view1.id)
        self.assertEqual(view, self.view1)

    @raises(Grouping.DoesNotExist)
    def test_get_single_view_with_contributor(self):
        Grouping.objects.get_single(
            self.contributor, self.project.id, self.view1.id)

    def test_get_single_view_with_view_user(self):
        view = Grouping.objects.get_single(
            self.view1_user, self.project.id, self.view1.id)
        self.assertEqual(view, self.view1)

    @raises(Grouping.DoesNotExist)
    def test_get_single_view_with_wrong_view_user(self):
        Grouping.objects.get_single(
            self.view2_user, self.project.id, self.view1.id)

    @raises(Project.DoesNotExist)
    def test_get_single_view_with_non_member(self):
        Grouping.objects.get_single(
            self.non_member, self.project.id, self.view1.id)

    def test_get_single_view_as_admin_with_admin(self):
        view = Grouping.objects.as_admin(
            self.admin, self.project.id, self.view1.id)
        self.assertEqual(view, self.view1)

    @raises(PermissionDenied)
    def test_get_single_view_as_admin_with_contributor(self):
        Grouping.objects.as_admin(
            self.contributor, self.project.id, self.view1.id)

    @raises(PermissionDenied)
    def test_get_single_view_as_admin_with_view_member(self):
        Grouping.objects.as_admin(
            self.view1_user, self.project.id, self.view1.id)

    @raises(Project.DoesNotExist)
    def test_get_single_view_as_admin_with_non_member(self):
        Grouping.objects.as_admin(
            self.non_member, self.project.id, self.view1.id)



import datetime
from django.test import TestCase

from geokey.categories.tests.model_factories import (
    TextFieldFactory, NumericFieldFactory, DateFieldFactory,
    LookupFieldFactory, LookupValueFactory, MultipleLookupValueFactory,
    MultipleLookupFieldFactory, CategoryFactory, DateTimeFieldFactory,
    TimeFieldFactory
)
from geokey.projects.tests.model_factories import ProjectF
from geokey.datagroupings.tests.model_factories import GroupingFactory
from geokey.users.tests.model_factories import (
    UserGroupF, GroupingUserGroupFactory
)

from geokey.datagroupings.tests.model_factories import RuleFactory

from ..templatetags.is_in import is_in
from ..templatetags.key import key, value, minval, maxval
from ..templatetags.filters import (
    get_textfield_filter, get_numericfield_filter, get_datefield_filter,
    get_singlelookup_filter, get_multiplelookup_filter, get_createdate_filter,
    filters
)
from ..templatetags.usergroups import get_user_group, usergroups


class TemplateTagsTest(TestCase):
    def test_is_in(self):
        dict = {
            'name': ["1", "2", "3"]
        }
        self.assertEqual(is_in(1, 'name', dict), 'selected')
        self.assertEqual(is_in(4, 'name', dict), '')

    def test_key(self):
        dict = {
            'name': 'Oliver'
        }
        self.assertEqual(key(dict, 'name'), 'Oliver')
        self.assertEqual(key(dict, 'blah'), '')

    def test_value(self):
        dict = {
            'name': 'Oliver'
        }
        self.assertEqual(value(dict, 'name'), 'Oliver')
        self.assertEqual(value(dict, 'blah'), '')

    def test_minval(self):
        dict = {
            'field': {"minval": 5}
        }
        self.assertEqual(minval(dict, 'field'), 5)
        self.assertEqual(minval(dict, 'blah'), '')

    def test_maxval(self):
        dict = {
            'field': {"maxval": 5}
        }
        self.assertEqual(maxval(dict, 'field'), 5)
        self.assertEqual(maxval(dict, 'blah'), '')


class FilterTest(TestCase):
    def test_get_textfield_filter(self):
        field = TextFieldFactory.create(**{'name': 'Field'})
        self.assertEqual(
            get_textfield_filter(field, 'blah'),
            '<li>Field contains blah</li>'
        )

    def test_get_numericfield_filter(self):
        field = NumericFieldFactory.create(**{'name': 'Field'})
        self.assertEqual(
            get_numericfield_filter(field, {"minval": 5}),
            '<li>Field is greater than 5</li>'
        )
        self.assertEqual(
            get_numericfield_filter(field, {"maxval": 5}),
            '<li>Field is lower than 5</li>'
        )
        self.assertEqual(
            get_numericfield_filter(field, {"minval": 1, "maxval": 5}),
            '<li>Field is greater than 1 and lower than 5</li>'
        )

    def test_get_datefield_filter(self):
        field = DateFieldFactory.create(**{'name': 'Field'})
        self.assertEqual(
            get_datefield_filter(field, {"minval": "2015-03-15"}),
            '<li>Field is after 2015-03-15</li>'
        )
        self.assertEqual(
            get_datefield_filter(field, {"maxval": "2015-03-31"}),
            '<li>Field is before 2015-03-31</li>'
        )
        self.assertEqual(
            get_datefield_filter(
                field, {"minval": "2015-03-15", "maxval": "2015-03-31"}),
            '<li>Field is after 2015-03-15 and before 2015-03-31</li>'
        )

    def test_get_singlelookup_filter(self):
        field = LookupFieldFactory.create(**{'name': 'Field'})
        val_1 = LookupValueFactory.create(**{'field': field})
        val_2 = LookupValueFactory.create(**{'field': field})
        LookupValueFactory.create(**{'field': field})

        self.assertEqual(
            get_singlelookup_filter(field, [val_1.id, val_2.id]),
            '<li>Field is one of %s, %s</li>' % (val_1.name, val_2.name)
        )

    def test_get_multiplelookup_filter(self):
        field = MultipleLookupFieldFactory.create(**{'name': 'Field'})
        val_1 = MultipleLookupValueFactory.create(**{'field': field})
        val_2 = MultipleLookupValueFactory.create(**{'field': field})
        MultipleLookupValueFactory.create(**{'field': field})

        self.assertEqual(
            get_multiplelookup_filter(field, [val_1.id, val_2.id]),
            '<li>Field matches at least one of %s, %s</li>' % (
                val_1.name, val_2.name)
        )

    def test_get_createdate_filter(self):
        rule = RuleFactory.create(
            **{'min_date': datetime.datetime(2007, 12, 5, 12, 00)}
        )
        self.assertEqual(
            '<li>the contribution has been created after '
            'Dec 05 2007 12:00</li>',
            get_createdate_filter(rule)
        )

        rule = RuleFactory.create(
            **{'max_date': datetime.datetime(2012, 12, 5, 15, 00)}
        )
        self.assertEqual(
            '<li>the contribution has been created before '
            'Dec 05 2012 15:00</li>',
            get_createdate_filter(rule)
        )

        rule = RuleFactory.create(**{
            'min_date': datetime.datetime(2007, 12, 5, 12, 00),
            'max_date': datetime.datetime(2012, 12, 5, 15, 00)
        })
        self.assertEqual(
            '<li>the contribution has been created after '
            'Dec 05 2007 12:00 and before Dec 05 2012 15:00</li>',
            get_createdate_filter(rule)
        )

    def test_filters(self):
        category = CategoryFactory.create()
        TextFieldFactory.create(**{'category': category, 'key': 'text'})
        NumericFieldFactory.create(**{'category': category, 'key': 'number'})
        DateFieldFactory.create(**{'category': category, 'key': 'date'})
        DateTimeFieldFactory.create(
            **{'category': category, 'key': 'datetime'})
        TimeFieldFactory.create(**{'category': category, 'key': 'time'})
        lookup = LookupFieldFactory.create(
            **{'category': category, 'key': 'lookup'})
        val_1 = LookupValueFactory.create(**{'field': lookup})
        multiple = MultipleLookupFieldFactory.create(
            **{'category': category, 'key': 'multiple'})
        mul_1 = MultipleLookupValueFactory.create(**{'field': multiple})

        rule = RuleFactory.create(**{
            'category': category,
            'min_date': datetime.datetime(2007, 12, 5, 12, 00),
            'max_date': datetime.datetime(2012, 12, 5, 15, 00),
            'constraints': {
                'text': 'blah',
                'number': {'minval': 0, 'maxval': 10},
                'date': {'minval': 0, 'maxval': 10},
                'datetime': {'minval': 0, 'maxval': 10},
                'time': {'minval': 0, 'maxval': 10},
                'lookup': [val_1.id],
                'multiple': [mul_1.id]
            }
        })

        result = filters(rule)
        self.assertEqual(result.count('<li>'), 8)


class UserGroupsTest(TestCase):
    def test_get_user_group(self):
        project = ProjectF.create()
        grouping = GroupingFactory.create(**{'project': project})
        group = UserGroupF(**{'project': project})

        html = get_user_group(group, grouping)
        self.assertEqual(
            html,
            '<li>'
            '<button type="button" name="%s" class="btn btn-default '
            'pull-right grant-single" data-toggle="button" ><span '
            'class="text-success">Grant access</span></button>'
            '<strong>%s</strong><p>%s</p>'
            '</li>' % (group.id, group.name, group.description)
        )

        GroupingUserGroupFactory.create(**{
            'usergroup': group,
            'grouping': grouping
        })

        html = get_user_group(group, grouping)
        self.assertEqual(
            html,
            '<li>'
            '<button type="button" name="%s" class="btn btn-default '
            'pull-right active grant-single" data-toggle="button" >'
            '<span class="text-danger">Revoke access</span></button>'
            '<strong>%s</strong><p>%s</p>'
            '</li>' % (group.id, group.name, group.description)
        )

        project.isprivate = False
        project.save()
        grouping.isprivate = False
        grouping.save()

        html = get_user_group(group, grouping)
        self.assertEqual(
            html,
            '<li>'
            '<button type="button" name="%s" class="btn btn-default '
            'pull-right active grant-single" data-toggle="button" '
            'disabled="disabled">'
            '<span class="text-danger">Revoke access</span></button>'
            '<strong>%s</strong><p>%s</p>'
            '</li>' % (group.id, group.name, group.description)
        )

    def test_usergroups(self):
        project = ProjectF.create()
        grouping = GroupingFactory.create(**{'project': project})
        group = UserGroupF(**{'project': project})

        html = usergroups(grouping)
        self.assertEqual(
            html,
            '<ul class="list-unstyled overview-list"><li>'
            '<button type="button" name="%s" class="btn btn-default '
            'pull-right grant-single" data-toggle="button" ><span '
            'class="text-success">Grant access</span></button>'
            '<strong>%s</strong><p>%s</p>'
            '</li></ul>' % (group.id, group.name, group.description)
        )



import json

from django.views.generic import CreateView, TemplateView
from django.core.urlresolvers import reverse
from django.shortcuts import redirect
from django.contrib import messages
from django.utils.html import strip_tags

from braces.views import LoginRequiredMixin

from geokey.core.decorators import handle_exceptions_for_admin

from geokey.projects.models import Project
from geokey.categories.models import Category

from .base import STATUS
from .forms import GroupingCreateForm
from .models import Grouping, Rule


# ############################################################################
#
# Administration views
#
# ############################################################################

class GroupingList(LoginRequiredMixin, TemplateView):
    template_name = 'datagroupings/grouping_list.html'

    @handle_exceptions_for_admin
    def get_context_data(self, project_id):
        """
        Returns the context to render the view. Overwrites the method to add
        the project to the context.

        Parameter
        ---------
        project_id : int
            identifies the project in the database

        Returns
        -------
        dict
            context
        """
        user = self.request.user

        context = super(GroupingList, self).get_context_data()
        context['project'] = Project.objects.as_admin(user, project_id)

        return context


class GroupingCreate(LoginRequiredMixin, CreateView):
    """
    Displays the create view page
    """
    form_class = GroupingCreateForm
    template_name = 'datagroupings/grouping_create.html'

    def get_success_url(self):
        """
        Returns the URL that is called after successfully creating the
        grouping.

        Returns
        -------
        str
            URL that is called
        """
        project_id = self.kwargs['project_id']
        return reverse(
            'admin:grouping_overview',
            kwargs={'project_id': project_id, 'grouping_id': self.object.id}
        )

    @handle_exceptions_for_admin
    def get_context_data(self, **kwargs):
        """
        Returns the context to render the view. Overwrites the method to add
        the project to the context.

        Returns
        -------
        dict
            context
        """
        project_id = self.kwargs['project_id']

        context = super(
            GroupingCreate, self).get_context_data(**kwargs)

        context['project'] = Project.objects.as_admin(
            self.request.user, project_id
        )
        return context

    def form_valid(self, form):
        """
        Is called when the POSTed data is valid and creates the observation
        type.

        Parameters
        ----------
        form : geokey.grouings.forms.GroupingCreateForm
            Represents the user input
        """
        project_id = self.kwargs['project_id']
        project = Project.objects.as_admin(self.request.user, project_id)

        form.instance.project = project
        form.instance.creator = self.request.user
        messages.success(self.request, "The data grouping has been created.")
        return super(GroupingCreate, self).form_valid(form)


class GroupingOverview(LoginRequiredMixin, TemplateView):
    """
    Provides an overview of all rules assigned to the grouping.
    """
    template_name = 'datagroupings/grouping_overview.html'

    @handle_exceptions_for_admin
    def get_context_data(self, project_id, grouping_id):
        """
        Returns the context to render the view. Overwrites the method to add
        the grouping to the context.

        Parameter
        ---------
        project_id : int
            identifies the project in the database
        grouping_id : int
            identifies the data grouping in the database

        Returns
        -------
        dict
            context
        """
        user = self.request.user
        grouping = Grouping.objects.as_admin(user, project_id, grouping_id)
        return super(GroupingOverview, self).get_context_data(
            grouping=grouping
        )


class GroupingSettings(LoginRequiredMixin, TemplateView):
    """
    Displays the page to change the settings of the data grouping
    """
    template_name = 'datagroupings/grouping_settings.html'

    @handle_exceptions_for_admin
    def get_context_data(self, project_id, grouping_id):
        """
        Returns the context to render the view. Overwrites the method to add
        the grouping and available status types to the context.

        Parameter
        ---------
        project_id : int
            identifies the project in the database
        grouping_id : int
            identifies the data grouping in the database

        Returns
        -------
        dict
            context
        """
        user = self.request.user
        grouping = Grouping.objects.as_admin(user, project_id, grouping_id)
        return {'grouping': grouping, 'status_types': STATUS}

    def post(self, request, project_id, grouping_id):
        """
        Updates the settings of the data grouping

        Parameter
        ---------
        request : django.http.HttpRequest
            Object representing the request.
        project_id : int
            identifies the project in the database
        grouping_id : int
            identifies the data grouping in the database

        Returns
        -------
        django.http.HttpResponse
            Rendered template
        """
        context = self.get_context_data(project_id, grouping_id)
        grouping = context.pop('grouping', None)

        if grouping is not None:
            data = request.POST

            grouping.name = strip_tags(data.get('name'))
            grouping.description = strip_tags(data.get('description'))
            grouping.save()

            messages.success(self.request, "The data grouping has been updated.")
            context['grouping'] = grouping

        return self.render_to_response(context)


class GroupingPermissions(LoginRequiredMixin, TemplateView):
    """
    Displays the page to change the permissions for the data grouping.
    Permissions are change via Ajax
    """
    template_name = 'datagroupings/grouping_permissions.html'

    @handle_exceptions_for_admin
    def get_context_data(self, project_id, grouping_id):
        """
        Returns the context to render the view. Overwrites the method to add
        the grouping to the context.

        Parameter
        ---------
        project_id : int
            identifies the project in the database
        grouping_id : int
            identifies the data grouping in the database

        Returns
        -------
        dict
            context
        """
        user = self.request.user
        grouping = Grouping.objects.as_admin(user, project_id, grouping_id)
        return super(GroupingPermissions, self).get_context_data(
            grouping=grouping
        )


class GroupingDelete(LoginRequiredMixin, TemplateView):
    """
    Deletes a data grouping
    """
    template_name = 'base.html'

    @handle_exceptions_for_admin
    def get_context_data(self, project_id, grouping_id):
        """
        Returns the context to render the view. Overwrites the method to add
        the grouping to the context.

        Parameter
        ---------
        project_id : int
            identifies the project in the database
        grouping_id : int
            identifies the data grouping in the database

        Returns
        -------
        dict
            context
        """
        user = self.request.user
        grouping = Grouping.objects.as_admin(user, project_id, grouping_id)
        return super(GroupingDelete, self).get_context_data(grouping=grouping)

    def get(self, request, project_id, grouping_id):
        """
        Deletes a data grouping

        Parameter
        ---------
        request : django.http.HttpRequest
            Object representing the request.
        project_id : int
            identifies the project in the database
        grouping_id : int
            identifies the data grouping in the database

        Returns
        -------
        django.http.HttpResponseRedirect
            redirecting to list of data groupings overview.

        django.http.HttpResponse
            If user is not administrator of the project, the error message is
            rendered.
        """
        context = self.get_context_data(project_id, grouping_id)
        grouping = context.pop('grouping', None)

        if grouping is not None:
            grouping.delete()
            messages.success(
                self.request,
                'The data grouping has been deleted'
            )

            return redirect('admin:grouping_list', project_id=project_id)

        return self.render_to_response(context)


class RuleCreate(LoginRequiredMixin, CreateView):
    """
    Displays the rule create page
    """
    template_name = 'datagroupings/rule_create.html'
    model = Rule

    @handle_exceptions_for_admin
    def get_context_data(self, *args, **kwargs):
        """
        Returns the context to render the view. Overwrites the method to add
        the grouping to the context.

        Returns
        -------
        dict
            context
        """
        self.object = None

        project_id = self.kwargs['project_id']
        grouping_id = self.kwargs['grouping_id']

        context = super(
            RuleCreate, self).get_context_data(*args, **kwargs)

        context['grouping'] = Grouping.objects.as_admin(
            self.request.user, project_id, grouping_id
        )
        return context

    def post(self, request, project_id, grouping_id):
        """
        Creates a new Rule with the POSTed data.

        Parameter
        ---------
        request : django.http.HttpRequest
            Object representing the request.
        project_id : int
            identifies the project in the database
        grouping_id : int
            identifies the data grouping in the database

        Returns
        -------
        django.http.HttpResponseRedirect
            redirecting to the rule settings.

        django.http.HttpResponse
            If user is administrator of the project, the error message is
            rendered.
        """
        context = self.get_context_data()
        grouping = context.pop('grouping', None)

        if grouping is not None:
            category = Category.objects.as_admin(
                self.request.user, project_id, request.POST.get('category'))

            rules = None
            min_date = None
            max_date = None

            rules = request.POST.get('rules', None)
            if rules:
                rules = json.loads(rules)
                min_date = rules.pop('min_date', None)
                max_date = rules.pop('max_date', None)

            Rule.objects.create(
                grouping=grouping,
                category=category,
                min_date=min_date,
                max_date=max_date,
                constraints=rules
            )
            messages.success(self.request, 'The filter has been created.')
            return redirect('admin:grouping_overview',
                            project_id=project_id, grouping_id=grouping_id)

        return self.render_to_response(context)


class RuleSettings(LoginRequiredMixin, TemplateView):
    """
    Displays the rule settings page
    """
    template_name = 'datagroupings/rule_settings.html'

    @handle_exceptions_for_admin
    def get_context_data(self, project_id, grouping_id, rule_id, **kwargs):
        """
        Returns the context to render the view. Overwrites the method to add
        the rule to the context.

        Parameter
        ---------
        project_id : int
            identifies the project in the database
        grouping_id : int
            identifies the data grouping in the database
        rule_id : int
            identifies the rule in the database

        Returns
        -------
        dict
            context
        """
        context = super(RuleSettings, self).get_context_data(**kwargs)

        view = Grouping.objects.as_admin(
            self.request.user,
            project_id,
            grouping_id
        )

        context['rule'] = view.rules.get(pk=rule_id)
        return context

    @handle_exceptions_for_admin
    def post(self, request, project_id, grouping_id, rule_id):
        """
        Updates the rule.

        Parameter
        ---------
        request : django.http.HttpRequest
            Object representing the request.
        project_id : int
            identifies the project in the database
        grouping_id : int
            identifies the data grouping in the database
        rule_id : int
            identifies the rule in the database

        Returns
        -------
        django.http.HttpResponse
            Rendered template
        """
        context = self.get_context_data(project_id, grouping_id, rule_id)

        rule = context.pop('rule', None)

        if rule is not None:
            rules = request.POST.get('rules', None)
            if rules:
                rules = json.loads(rules)
                rule.min_date = rules.pop('min_date', None)
                rule.max_date = rules.pop('max_date', None)
                rule.constraints = rules

            rule.save()
            messages.success(self.request, 'The filter has been updated.')
            return redirect('admin:grouping_overview',
                            project_id=project_id, grouping_id=grouping_id)

        return self.render_to_response(context)


class RuleDelete(LoginRequiredMixin, TemplateView):
    """
    Deletes the rule
    """
    template_name = 'base.html'

    @handle_exceptions_for_admin
    def get_context_data(self, project_id, grouping_id, rule_id, **kwargs):
        """
        Returns the context to render the view. Overwrites the method to add
        the rule to the context.

        Parameter
        ---------
        project_id : int
            identifies the project in the database
        grouping_id : int
            identifies the data grouping in the database
        rule_id : int
            identifies the rule in the database

        Returns
        -------
        dict
            context
        """
        grouping = Grouping.objects.as_admin(
            self.request.user, project_id, grouping_id)
        context = super(RuleDelete, self).get_context_data(**kwargs)

        context['rule'] = grouping.rules.get(pk=rule_id)
        return context

    def get(self, request, project_id, grouping_id, rule_id):
        """
        Deletes the rule

        Parameter
        ---------
        request : django.http.HttpRequest
            Object representing the request.
        project_id : int
            identifies the project in the database
        grouping_id : int
            identifies the data grouping in the database
        rule_id : int
            identifies the rule in the database

        Returns
        -------
        django.http.HttpResponseRedirect
            redirecting to list of rules overview.

        django.http.HttpResponse
            If user is administrator of the project, the error message is
            rendered.
        """
        context = self.get_context_data(project_id, grouping_id, rule_id)
        rule = context.pop('rule', None)

        if rule is not None:
            rule.delete()

            messages.success(self.request, 'The filter has been deleted')
            return redirect(
                'admin:grouping_overview',
                project_id=project_id, grouping_id=grouping_id
            )

        return self.render_to_response(context)



from django import template

register = template.Library()


def get_textfield_filter(field, rule_filter):
    return '<li>%s contains %s</li>' % (field.name, rule_filter)


def get_numericfield_filter(field, rule_filter):
    minval = rule_filter.get('minval')
    maxval = rule_filter.get('maxval')

    filter_string = '<li>%s is ' % (field.name)

    if minval is not None:
        filter_string = filter_string + 'greater than %s' % minval
    if minval is not None and maxval is not None:
        filter_string = filter_string + ' and '
    if maxval is not None:
        filter_string = filter_string + 'lower than %s' % maxval
    filter_string = filter_string + '</li>'

    return filter_string


def get_datefield_filter(field, rule_filter):
    minval = rule_filter.get('minval')
    maxval = rule_filter.get('maxval')

    filter_string = '<li>%s is ' % (field.name)

    if minval is not None:
        filter_string = filter_string + 'after %s' % minval
    if minval is not None and maxval is not None:
        filter_string = filter_string + ' and '
    if maxval is not None:
        filter_string = filter_string + 'before %s' % maxval
    filter_string = filter_string + '</li>'

    return filter_string


def get_singlelookup_filter(field, rule_filter):
    values = []
    for lookup in rule_filter:
        values.append(field.lookupvalues.get(pk=lookup).name)

    return '<li>%s is one of %s</li>' % (
        field.name, ', '.join(values)
    )


def get_multiplelookup_filter(field, rule_filter):
    values = []
    for lookup in rule_filter:
        values.append(field.lookupvalues.get(pk=lookup).name)

    return '<li>%s matches at least one of %s</li>' % (
        field.name, ', '.join(values)
    )


def get_createdate_filter(rule):
    filter_string = '<li>the contribution has been created '
    if rule.min_date is not None:
        filter_string = filter_string + 'after %s' % (
            rule.min_date.strftime("%b %d %Y %H:%M"))
    if rule.min_date is not None and rule.max_date is not None:
        filter_string = filter_string + ' and '
    if rule.max_date is not None:
        filter_string = filter_string + 'before %s' % (
            rule.max_date.strftime("%b %d %Y %H:%M"))
    filter_string = filter_string + '</li>'

    return filter_string


@register.simple_tag
def filters(rule):
    s = ''

    if rule.min_date is not None or rule.max_date is not None:
        s = s + get_createdate_filter(rule)

    for field in rule.category.fields.all():
        rule_filter = None
        if rule.constraints is not None:
            rule_filter = rule.constraints.get(field.key)

        if rule_filter is not None:
            if field.fieldtype == 'TextField':
                s = s + get_textfield_filter(field, rule_filter)

            elif field.fieldtype == 'NumericField':
                s = s + get_numericfield_filter(field, rule_filter)

            elif (field.fieldtype == 'DateTimeField' or
                    field.fieldtype == 'DateField' or
                    field.fieldtype == 'TimeField'):
                s = s + get_datefield_filter(field, rule_filter)

            elif field.fieldtype == 'LookupField':
                s = s + get_singlelookup_filter(field, rule_filter)

            elif field.fieldtype == 'MultipleLookupField':
                s = s + get_multiplelookup_filter(field, rule_filter)

    return s



from django.db import models
from django.db.models import Q

from geokey.projects.models import Project

from .base import STATUS


class GroupingQuerySet(models.query.QuerySet):
    """
    Queryset Manager for DataGroupings model
    """
    def for_user(self, user):
        """
        Returns all groupings accessable by the user.

        Parameter
        ---------
        user : geokey.users.models.User
            User the groupings are queried for

        Returns
        -------
        django.db.models.query.QuerySet
            List of data groupings
        """
        if user.is_anonymous():
            return self.filter(
                status='active', isprivate=False, project__isprivate=False)
        else:
            return self.filter(Q(status='active') & (
                               Q(isprivate=False, project__isprivate=False) |
                               Q(project__admins=user) |
                               Q(usergroups__usergroup__users=user))
                               ).distinct()


class GroupingManager(models.Manager):
    def get_queryset(self):
        """
        Returns all groupings excluding those with status deleted.

        Returns
        -------
        django.db.models.query.QuerySet
            List of data groupings
        """
        return GroupingQuerySet(self.model).exclude(status=STATUS.deleted)

    def for_user(self, user):
        """
        Returns all groupings accessable by the user.

        Parameter
        ---------
        user : geokey.users.models.User
            User the groupings are queried for

        Returns
        -------
        django.db.models.query.QuerySet
            List of data groupings
        """
        return self.get_queryset().for_user(user)

    def get_list(self, user, project_id):
        """
        Returns all groupings accessable by the user in the given project.

        Parameter
        ---------
        user : geokey.users.models.User
            User the groupings are queried for
        project_id : int
            identifies the project in the database

        Returns
        -------
        django.db.models.query.QuerySet
            List of data groupings
        """
        project = Project.objects.get_single(user, project_id)
        return project.groupings.for_user(user)

    def get_single(self, user, project_id, grouping_id):
        """
        Returns a single groupings from the given project, if accessable by the
        user.

        Parameter
        ---------
        user : geokey.users.models.User
            User the grouping is queried for
        project_id : int
            identifies the project in the database
        grouping_id : int
            identifies the data grouping in the database

        Returns
        -------
        geokey.datagroupings.models.DataGrouping
        """
        return self.get_list(user, project_id).get(pk=grouping_id)

    def as_admin(self, user, project_id, grouping_id):
        """
        Returns a single groupings from the given project, if the user is admin
        of the project.

        Parameter
        ---------
        user : geokey.users.models.User
            User the grouping is queried for
        project_id : int
            identifies the project in the database
        grouping_id : int
            identifies the data grouping in the database

        Returns
        -------
        geokey.datagroupings.models.DataGrouping
        """
        project = Project.objects.as_admin(user, project_id)
        return project.groupings.get(pk=grouping_id)


class RuleManager(models.Manager):
    """
    Queryset Manager for Rule model
    """
    def get_queryset(self):
        """
        Returns all rules excluding deleted ones.

        Returns
        -------
        django.db.models.query.QuerySet
            List of rules
        """
        return super(RuleManager, self).get_queryset().exclude(
            status=STATUS.deleted)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('datagroupings', '0005_auto_20150312_1741'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='rule',
            name='filters',
        ),
    ]



from django import template

register = template.Library()


@register.simple_tag
def is_in(value, key, d):
    if d is not None:
        if d.get(key) is not None:
            if str(value) in d[key]:
                return 'selected'

    return ''



import json

from django.test import TestCase
from django.core.urlresolvers import reverse
from django.contrib.auth.models import AnonymousUser
from django.http import HttpResponseRedirect
from django.contrib.gis.geos import GEOSGeometry
from django.core import mail

from nose.tools import raises

from rest_framework.test import APIRequestFactory, force_authenticate

from geokey.datagroupings.tests.model_factories import GroupingFactory
from geokey.categories.tests.model_factories import (
    TextFieldFactory, CategoryFactory
)

from .model_factories import UserF, ProjectF
from ..models import Project
from ..views import (
    ProjectCreate, ProjectSettings, ProjectUpdate, ProjectAdmins,
    ProjectAdminsUser, Projects, SingleProject, ProjectOverview, ProjectExtend,
    ProjectContactAdmins, CategoriesReorderView, ProjectsInvolved,
    ProjectDelete
)

# ############################################################################
#
# ADMIN VIEWS
#
# ############################################################################


class ProjectCreateTest(TestCase):
    def test_get_with_user(self):
        view = ProjectCreate.as_view()
        url = reverse('admin:project_create')
        request = APIRequestFactory().get(url)
        request.user = UserF.create()
        response = view(request).render()
        self.assertEqual(response.status_code, 200)

    def test_get_with_anonymous(self):
        view = ProjectCreate.as_view()
        url = reverse('admin:project_create')
        request = APIRequestFactory().get(url)
        request.user = AnonymousUser()
        response = view(request)
        self.assertTrue(isinstance(response, HttpResponseRedirect))

    def test_post_with_user(self):
        data = {
            'name': 'Project',
            'description': '',
            'isprivate': True,
            'everyone_contributes': 'auth'
        }
        view = ProjectCreate.as_view()
        url = reverse('admin:project_create')
        request = APIRequestFactory().post(url, data)
        request.user = UserF.create()

        from django.contrib.messages.storage.fallback import FallbackStorage
        setattr(request, 'session', 'session')
        messages = FallbackStorage(request)
        setattr(request, '_messages', messages)

        response = view(request)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(Project.objects.count(), 1)

    def test_post_with_anonymous(self):
        data = {
            'name': 'Project',
            'description': '',
            'isprivate': True,
            'everyone_contributes': 'auth'
        }
        view = ProjectCreate.as_view()
        url = reverse('admin:project_create')
        request = APIRequestFactory().post(url, data)
        request.user = AnonymousUser()

        response = view(request)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(Project.objects.count(), 0)


class ProjectsInvolvedTest(TestCase):
    def test_get_with_user(self):
        user = UserF.create()
        ProjectF.create(add_contributors=[user])
        view = ProjectsInvolved.as_view()
        url = reverse('admin:projects_involved')
        request = APIRequestFactory().get(url)
        request.user = user
        response = view(request).render()
        self.assertEqual(response.status_code, 200)

    def test_get_with_anonymous(self):
        view = ProjectsInvolved.as_view()
        url = reverse('admin:projects_involved')
        request = APIRequestFactory().get(url)
        request.user = AnonymousUser()
        response = view(request)
        self.assertEqual(response.status_code, 302)


class ProjectExtendTest(TestCase):
    def setUp(self):
        self.creator = UserF.create()
        self.admin = UserF.create()
        self.view_member = UserF.create()
        self.contributor = UserF.create()
        self.project = ProjectF.create(
            add_admins=[self.admin],
            add_contributors=[self.contributor],
            add_viewers=[self.view_member],
            **{
                'creator': self.creator
            }
        )

    def test_get_with_creator(self):
        view = ProjectSettings.as_view()
        url = reverse('admin:project_extend',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().get(url)
        request.user = self.creator
        response = view(request, project_id=self.project.id).render()
        self.assertEqual(response.status_code, 200)

    def test_get_with_admin(self):
        view = ProjectExtend.as_view()
        url = reverse('admin:project_extend',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().get(url)
        request.user = self.admin
        response = view(request, project_id=self.project.id).render()
        self.assertEqual(response.status_code, 200)

    def test_get_with_contributor(self):
        view = ProjectExtend.as_view()
        url = reverse('admin:project_extend',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().get(url)
        request.user = self.contributor
        response = view(request, project_id=self.project.id)
        self.assertEqual(response.status_code, 200)

    def test_get_with_view_member(self):
        view = ProjectExtend.as_view()
        url = reverse(
            'admin:project_extend',
            kwargs={'project_id': self.project.id}
        )
        request = APIRequestFactory().get(url)
        request.user = self.view_member
        response = view(request, project_id=self.project.id)
        self.assertEqual(response.status_code, 200)

    def test_get_with_anonymous(self):
        view = ProjectExtend.as_view()
        url = reverse(
            'admin:project_extend',
            kwargs={'project_id': self.project.id}
        )
        request = APIRequestFactory().get(url)
        request.user = AnonymousUser()
        response = view(request)
        self.assertTrue(isinstance(response, HttpResponseRedirect))

    def test_get_deleted_project(self):
        self.project.delete()
        view = ProjectExtend.as_view()
        url = reverse('admin:project_extend',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().get(url)
        request.user = self.admin
        response = view(request, project_id=self.project.id).render()
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Project matching query does not exist.')

    def test_update(self):
        data = {'geometry': '{"type": "Polygon","coordinates": [['
                            '[-0.508,51.682],[-0.53,51.327],[0.225,51.323],'
                            '[0.167,51.667],[-0.508,51.682]]]}'}
        view = ProjectExtend.as_view()
        url = reverse('admin:project_extend',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().post(url, data)
        request.user = self.admin

        from django.contrib.messages.storage.fallback import FallbackStorage
        setattr(request, 'session', 'session')
        messages = FallbackStorage(request)
        setattr(request, '_messages', messages)

        response = view(request, project_id=self.project.id).render()
        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response, 'The geographic extent has been updated successfully.')

        updated = Project.objects.get(pk=self.project.id)

        self.assertEqual(updated.geographic_extend.geom_type, 'Polygon')
        self.assertEqual(
            updated.geographic_extend.json,
            GEOSGeometry(data.get('geometry')).json
        )

    def test_update_with_none(self):
        data = None
        view = ProjectExtend.as_view()
        url = reverse('admin:project_extend',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().post(url, data)
        request.user = self.admin

        from django.contrib.messages.storage.fallback import FallbackStorage
        setattr(request, 'session', 'session')
        messages = FallbackStorage(request)
        setattr(request, '_messages', messages)

        response = view(request, project_id=self.project.id).render()
        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response, 'The geographic extent has been updated successfully.')

        updated = Project.objects.get(pk=self.project.id)

        self.assertEqual(updated.geographic_extend, None)

    def test_update_with_empty_string(self):
        data = {'geometry': ''}
        view = ProjectExtend.as_view()
        url = reverse('admin:project_extend',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().post(url, data)
        request.user = self.admin

        from django.contrib.messages.storage.fallback import FallbackStorage
        setattr(request, 'session', 'session')
        messages = FallbackStorage(request)
        setattr(request, '_messages', messages)

        response = view(request, project_id=self.project.id).render()
        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response, 'The geographic extent has been updated successfully.')

        updated = Project.objects.get(pk=self.project.id)

        self.assertEqual(updated.geographic_extend, None)


class ProjectSettingsTest(TestCase):
    def setUp(self):
        self.creator = UserF.create()
        self.admin = UserF.create()
        self.view_member = UserF.create()
        self.contributor = UserF.create()
        self.project = ProjectF.create(
            add_admins=[self.admin],
            add_contributors=[self.contributor],
            add_viewers=[self.view_member],
            **{
                'creator': self.creator
            }
        )

    def test_get_with_creator(self):
        view = ProjectSettings.as_view()
        url = reverse('admin:project_settings',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().get(url)
        request.user = self.creator
        response = view(request, project_id=self.project.id).render()
        self.assertEqual(response.status_code, 200)

    def test_get_with_admin(self):
        view = ProjectSettings.as_view()
        url = reverse('admin:project_settings',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().get(url)
        request.user = self.admin
        response = view(request, project_id=self.project.id).render()
        self.assertEqual(response.status_code, 200)

    def test_get_with_contributor(self):
        view = ProjectSettings.as_view()
        url = reverse('admin:project_settings',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().get(url)
        request.user = self.contributor
        response = view(request, project_id=self.project.id)
        self.assertEqual(response.status_code, 200)

    def test_get_with_view_member(self):
        view = ProjectSettings.as_view()
        url = reverse('admin:project_settings',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().get(url)
        request.user = self.view_member
        response = view(request, project_id=self.project.id)
        self.assertEqual(response.status_code, 200)

    def test_get_with_anonymous(self):
        view = ProjectSettings.as_view()
        url = reverse(
            'admin:project_extend',
            kwargs={'project_id': self.project.id}
        )
        request = APIRequestFactory().get(url)
        request.user = AnonymousUser()
        response = view(request)
        self.assertTrue(isinstance(response, HttpResponseRedirect))

    def test_get_deleted_project(self):
        self.project.delete()
        view = ProjectSettings.as_view()
        url = reverse('admin:project_settings',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().get(url)
        request.user = self.admin
        response = view(request, project_id=self.project.id).render()
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Project matching query does not exist.')

    def test_post_with_admin(self):
        data = {
            'name': 'Project',
            'description': '',
            'isprivate': True,
            'everyone_contributes': 'auth'
        }
        view = ProjectSettings.as_view()
        url = reverse('admin:project_settings',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().post(url, data)
        request.user = self.creator

        from django.contrib.messages.storage.fallback import FallbackStorage
        setattr(request, 'session', 'session')
        messages = FallbackStorage(request)
        setattr(request, '_messages', messages)

        response = view(request, project_id=self.project.id).render()
        self.assertEqual(response.status_code, 200)


class ProjectOverviewTest(TestCase):
    def setUp(self):
        self.creator = UserF.create()
        self.admin = UserF.create()
        self.view_member = UserF.create()
        self.contributor = UserF.create()
        self.project = ProjectF.create(
            add_admins=[self.admin],
            add_contributors=[self.contributor],
            add_viewers=[self.view_member],
            **{
                'creator': self.creator
            }
        )

    def test_get_with_admin(self):
        view = ProjectOverview.as_view()
        url = reverse('admin:project_overview',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().get(url)
        request.user = self.admin
        response = view(request, project_id=self.project.id).render()
        self.assertEqual(response.status_code, 200)

    def test_get_with_contributor(self):
        view = ProjectOverview.as_view()
        url = reverse('admin:project_overview',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().get(url)
        request.user = self.contributor
        response = view(request, project_id=self.project.id)
        self.assertEqual(response.status_code, 200)

    def test_get_with_view_member(self):
        view = ProjectOverview.as_view()
        url = reverse('admin:project_overview',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().get(url)
        request.user = self.view_member
        response = view(request, project_id=self.project.id)
        self.assertEqual(response.status_code, 200)

    def test_get_with_anonymous(self):
        view = ProjectOverview.as_view()
        url = reverse('admin:project_overview',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().get(url)
        request.user = AnonymousUser()
        response = view(request)
        self.assertTrue(isinstance(response, HttpResponseRedirect))

    def test_get_deleted_project(self):
        self.project.delete()
        view = ProjectOverview.as_view()
        url = reverse('admin:project_overview',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().get(url)
        request.user = self.admin
        response = view(request, project_id=self.project.id).render()
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Project matching query does not exist.')


class ProjectDeleteTest(TestCase):
    def test_delete_with_admin(self):
        user = UserF.create()
        project = ProjectF.create(add_admins=[user])

        view = ProjectDelete.as_view()
        url = reverse('admin:project_delete',
                      kwargs={'project_id': project.id})
        request = APIRequestFactory().get(url)
        request.user = user

        from django.contrib.messages.storage.fallback import FallbackStorage
        setattr(request, 'session', 'session')
        messages = FallbackStorage(request)
        setattr(request, '_messages', messages)

        response = view(request, project_id=project.id)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(Project.objects.count(), 0)

    def test_delete_with_contributor(self):
        user = UserF.create()
        project = ProjectF.create(add_contributors=[user])

        view = ProjectDelete.as_view()
        url = reverse('admin:project_delete',
                      kwargs={'project_id': project.id})
        request = APIRequestFactory().get(url)
        request.user = user
        response = view(request, project_id=project.id)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(Project.objects.count(), 1)

    def test_delete_with_anonymous(self):
        project = ProjectF.create()

        view = ProjectDelete.as_view()
        url = reverse('admin:project_delete',
                      kwargs={'project_id': project.id})
        request = APIRequestFactory().get(url)
        request.user = AnonymousUser()
        response = view(request, project_id=project.id)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(Project.objects.count(), 1)


# ############################################################################
#
# AJAX VIEWS
#
# ############################################################################

class ProjectUpdateTest(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserF.create()
        self.contributor = UserF.create()
        self.non_member = UserF.create()

        self.project = ProjectF.create(
            add_admins=[self.admin],
            add_contributors=[self.contributor]
        )

    def test_unauthenticated(self):
        request = self.factory.put(
            '/api/projects/%s/' % self.project.id,
            {'status': 'bockwurst'}
        )
        view = ProjectUpdate.as_view()
        response = view(request, project_id=self.project.id).render()

        self.assertEqual(response.status_code, 404)

    def test_update_with_wrong_status(self):
        request = self.factory.put(
            '/api/projects/%s/' % self.project.id,
            {'status': 'bockwurst'}
        )
        force_authenticate(request, user=self.admin)
        view = ProjectUpdate.as_view()
        response = view(request, project_id=self.project.id).render()

        self.assertEqual(response.status_code, 400)

    def test_update_project_status_with_admin(self):
        request = self.factory.put(
            '/api/projects/%s/' % self.project.id,
            {'status': 'inactive'}
        )
        force_authenticate(request, user=self.admin)
        view = ProjectUpdate.as_view()
        response = view(request, project_id=self.project.id).render()

        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            Project.objects.get(pk=self.project.id).status,
            'inactive'
        )

    def test_update_project_description_with_admin(self):
        request = self.factory.put(
            '/api/projects/%s/' % self.project.id,
            {'description': 'new description'}
        )
        force_authenticate(request, user=self.admin)
        view = ProjectUpdate.as_view()
        response = view(request, project_id=self.project.id).render()

        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            Project.objects.get(pk=self.project.id).description,
            'new description'
        )

    def test_update_project_description_with_contributor(self):
        request = self.factory.put(
            '/api/projects/%s/' % self.project.id,
            {'description': 'new description'}
        )
        force_authenticate(request, user=self.contributor)
        view = ProjectUpdate.as_view()
        response = view(request, project_id=self.project.id).render()

        self.assertEqual(response.status_code, 403)
        self.assertEqual(
            Project.objects.get(pk=self.project.id).description,
            self.project.description
        )

    def test_update_project_description_with_non_member(self):
        request = self.factory.put(
            '/api/projects/%s/' % self.project.id,
            {'description': 'new description'}
        )
        force_authenticate(request, user=self.non_member)
        view = ProjectUpdate.as_view()
        response = view(request, project_id=self.project.id).render()

        self.assertEqual(response.status_code, 404)
        self.assertEqual(
            Project.objects.get(pk=self.project.id).description,
            self.project.description
        )


class ProjectAdminsTest(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserF.create()
        self.contributor = UserF.create()
        self.non_member = UserF.create()
        self.user_to_add = UserF.create()

        self.project = ProjectF.create(
            add_admins=[self.admin],
            add_contributors=[self.contributor]
        )

    def test_add_not_existing_user(self):
        request = self.factory.post(
            '/ajax/projects/%s/admins/' % (self.project.id) + '/',
            {'userId': 468476351545643131}
        )
        force_authenticate(request, user=self.admin)
        view = ProjectAdmins.as_view()
        response = view(
            request,
            project_id=self.project.id
        ).render()

        self.assertEqual(response.status_code, 400)

    def test_add_admin_with_admin(self):
        request = self.factory.post(
            '/ajax/projects/%s/admins/' % (self.project.id) + '/',
            {'userId': self.user_to_add.id}
        )
        force_authenticate(request, user=self.admin)
        view = ProjectAdmins.as_view()
        response = view(
            request,
            project_id=self.project.id
        ).render()

        self.assertEqual(response.status_code, 201)
        self.assertIn(
            self.user_to_add,
            Project.objects.get(pk=self.project.id).admins.all()
        )

    def test_add_admin_with_contributor(self):
        request = self.factory.post(
            '/ajax/projects/%s/admins/' % (self.project.id) + '/',
            {'userId': self.user_to_add.id}
        )
        force_authenticate(request, user=self.contributor)
        view = ProjectAdmins.as_view()
        response = view(
            request,
            project_id=self.project.id
        ).render()

        self.assertEqual(response.status_code, 403)
        self.assertNotIn(
            self.user_to_add,
            Project.objects.get(pk=self.project.id).admins.all()
        )

    def test_add_admin_with_non_member(self):
        request = self.factory.post(
            '/ajax/projects/%s/admins/' % (self.project.id) + '/',
            {'userId': self.user_to_add.id}
        )
        force_authenticate(request, user=self.non_member)
        view = ProjectAdmins.as_view()
        response = view(
            request,
            project_id=self.project.id
        ).render()

        self.assertEqual(response.status_code, 404)
        self.assertNotIn(
            self.user_to_add,
            Project.objects.get(pk=self.project.id).admins.all()
        )


class ProjectAdminsUserTest(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserF.create()
        self.contributor = UserF.create()
        self.non_member = UserF.create()
        self.admin_to_remove = UserF.create()

        self.project = ProjectF.create(
            add_admins=[self.admin, self.admin_to_remove],
            add_contributors=[self.contributor]
        )

    def test_delete_not_existing_admin(self):
        user = UserF.create()
        request = self.factory.delete(
            '/ajax/projects/%s/admins/%s/' %
            (self.project.id, user.id)
        )
        force_authenticate(request, user=self.admin)
        view = ProjectAdminsUser.as_view()
        response = view(
            request,
            project_id=self.project.id,
            user_id=user.id
        ).render()
        self.assertEqual(response.status_code, 404)

    def test_delete_adminuser_with_admin(self):
        request = self.factory.delete(
            '/ajax/projects/%s/admins/%s/' %
            (self.project.id, self.admin_to_remove.id)
        )
        force_authenticate(request, user=self.admin)
        view = ProjectAdminsUser.as_view()
        response = view(
            request,
            project_id=self.project.id,
            user_id=self.admin_to_remove.id
        ).render()

        self.assertEqual(response.status_code, 204)
        self.assertNotIn(
            self.admin_to_remove,
            Project.objects.get(pk=self.project.id).admins.all()
        )

    def test_delete_adminuser_with_contributor(self):
        request = self.factory.delete(
            '/ajax/projects/%s/admins/%s/' %
            (self.project.id, self.admin_to_remove.id)
        )
        force_authenticate(request, user=self.contributor)
        view = ProjectAdminsUser.as_view()
        response = view(
            request,
            project_id=self.project.id,
            user_id=self.admin_to_remove.id
        ).render()

        self.assertEqual(response.status_code, 403)
        self.assertIn(
            self.admin_to_remove,
            Project.objects.get(pk=self.project.id).admins.all()
        )

    def test_delete_adminuser_with_non_member(self):
        request = self.factory.delete(
            '/ajax/projects/%s/admins/%s/' %
            (self.project.id, self.admin_to_remove.id)
        )
        force_authenticate(request, user=self.non_member)
        view = ProjectAdminsUser.as_view()
        response = view(
            request,
            project_id=self.project.id,
            user_id=self.admin_to_remove.id
        ).render()

        self.assertEqual(response.status_code, 404)
        self.assertIn(
            self.admin_to_remove,
            Project.objects.get(pk=self.project.id).admins.all()
        )


class ReorderCategoriesTest(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.project = ProjectF.create()

        self.category_0 = CategoryFactory.create(**{'project': self.project})
        self.category_1 = CategoryFactory.create(**{'project': self.project})
        self.category_2 = CategoryFactory.create(**{'project': self.project})
        self.category_3 = CategoryFactory.create(**{'project': self.project})
        self.category_4 = CategoryFactory.create(**{'project': self.project})

    def test_reorder(self):
        url = reverse(
            'ajax:categories_reorder',
            kwargs={
                'project_id': self.project.id
            }
        )

        data = [
            self.category_4.id, self.category_0.id, self.category_2.id,
            self.category_1.id, self.category_3.id
        ]

        request = self.factory.post(
            url, json.dumps({'order': data}), content_type='application/json')
        force_authenticate(request, user=self.project.creator)
        view = CategoriesReorderView.as_view()
        response = view(
            request,
            project_id=self.project.id
        ).render()

        self.assertEqual(response.status_code, 200)

        categories = self.project.categories.all()

        self.assertTrue(categories.ordered)
        self.assertEqual(categories[0], self.category_4)
        self.assertEqual(categories[1], self.category_0)
        self.assertEqual(categories[2], self.category_2)
        self.assertEqual(categories[3], self.category_1)
        self.assertEqual(categories[4], self.category_3)

    def test_reorder_with_false_category(self):
        url = reverse(
            'ajax:categories_reorder',
            kwargs={
                'project_id': self.project.id
            }
        )

        data = [
            self.category_4.id, self.category_0.id, self.category_2.id,
            self.category_1.id, 655123135135
        ]

        request = self.factory.post(
            url, json.dumps({'order': data}), content_type='application/json')
        force_authenticate(request, user=self.project.creator)
        view = CategoriesReorderView.as_view()
        response = view(
            request,
            project_id=self.project.id
        ).render()

        self.assertEqual(response.status_code, 400)

        categories = self.project.categories.all()
        self.assertTrue(categories.ordered)
        self.assertEqual(categories[0].order, 0)
        self.assertEqual(categories[1].order, 1)
        self.assertEqual(categories[2].order, 2)
        self.assertEqual(categories[3].order, 3)
        self.assertEqual(categories[4].order, 4)


# ############################################################################
#
# API VIEWS
#
# ############################################################################

class ProjectsTest(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserF.create()
        self.contributor = UserF.create()
        self.non_member = UserF.create()
        self.view_member = UserF.create()

        self.public_project = ProjectF.create(
            add_admins=[self.admin],
            add_contributors=[self.contributor],
            **{
                'isprivate': False
            }
        )

        GroupingFactory(
            add_viewers=[self.view_member],
            **{'project': self.public_project, 'isprivate': False}
        )

        self.private_project = ProjectF.create(
            add_admins=[self.admin],
            add_contributors=[self.contributor],
            add_viewers=[self.view_member]
        )

        self.inactive_project = ProjectF.create(
            add_admins=[self.admin],
            add_contributors=[self.contributor],
            add_viewers=[self.view_member],
            **{
                'status': 'inactive'
            }
        )

        self.deleted_project = ProjectF.create(
            add_admins=[self.admin],
            add_contributors=[self.contributor],
            add_viewers=[self.view_member],
            **{'isprivate': False}
        )
        self.deleted_project.delete()

    def test_get_projects_with_admin(self):
        request = self.factory.get('/api/projects/')
        force_authenticate(request, user=self.admin)
        view = Projects.as_view()
        response = view(request).render()
        projects = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(projects), 2)
        self.assertNotContains(response, self.deleted_project.name)

    def test_get_projects_with_contributor(self):
        request = self.factory.get('/api/projects/')
        force_authenticate(request, user=self.contributor)
        view = Projects.as_view()
        response = view(request).render()

        projects = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(projects), 2)
        self.assertNotContains(response, self.inactive_project.name)
        self.assertNotContains(response, self.deleted_project.name)

    def test_get_projects_with_view_member(self):
        request = self.factory.get('/api/projects/')
        force_authenticate(request, user=self.view_member)
        view = Projects.as_view()
        response = view(request).render()

        projects = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(projects), 2)
        self.assertNotContains(response, self.inactive_project.name)
        self.assertNotContains(response, self.deleted_project.name)

    def test_get_projects_with_non_member(self):
        request = self.factory.get('/api/projects/')
        force_authenticate(request, user=self.non_member)
        view = Projects.as_view()
        response = view(request).render()

        projects = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(projects), 1)
        self.assertNotContains(response, self.inactive_project.name)
        self.assertNotContains(response, self.deleted_project.name)
        self.assertNotContains(response, self.private_project.name)

    def test_get_projects_with_anonymous(self):
        request = self.factory.get('/api/projects/')
        force_authenticate(request, user=AnonymousUser())
        view = Projects.as_view()
        response = view(request).render()

        projects = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(projects), 1)
        self.assertNotContains(response, self.inactive_project.name)
        self.assertNotContains(response, self.deleted_project.name)
        self.assertNotContains(response, self.private_project.name)


class SingleProjectTest(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()

    def test_category_serialization(self):
        user = UserF.create()

        project = ProjectF.create(
            add_admins=[user]
        )
        CategoryFactory.create(**{'project': project})
        CategoryFactory.create(
            **{'project': project, 'status': 'inactive'}
        )
        o1 = CategoryFactory.create(**{'project': project})
        TextFieldFactory.create(**{'category': o1})
        o2 = CategoryFactory.create(**{'project': project})
        TextFieldFactory.create(**{'category': o2})

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            2,
            len(json.loads(response.content).get('categories'))
        )

    def test_get_deleted_project_with_admin(self):
        user = UserF.create()

        project = ProjectF.create(
            add_admins=[user]
        )
        project.delete()

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 404)

    def test_get_private_project_with_admin(self):
        user = UserF.create()

        project = ProjectF.create(
            add_admins=[user],
            add_viewers=[UserF.create()]
        )

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, project.name)
        self.assertContains(response, '"can_contribute":true')
        self.assertContains(response, '"is_involved":true')
        self.assertContains(response, '"num_contributions"')
        self.assertContains(response, '"geographic_extent"')

    def test_get_inactive_project_with_admin(self):
        user = UserF.create()

        project = ProjectF.create(
            add_admins=[user],
            **{'status': 'inactive'}
        )

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 403)

    def test_get_public_project_with_admin(self):
        user = UserF.create()

        project = ProjectF.create(
            add_admins=[user],
            **{'isprivate': False}
        )

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, project.name)
        self.assertContains(response, '"can_contribute":true')

    def test_get_deleted_project_with_contributor(self):
        user = UserF.create()

        project = ProjectF.create(
            add_contributors=[user]
        )
        project.delete()

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 404)

    def test_get_private_project_with_contributor(self):
        user = UserF.create()

        project = ProjectF.create(
            add_contributors=[user]
        )

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, project.name)
        self.assertContains(response, '"can_contribute":true')

    def test_get_inactive_project_with_contributor(self):
        user = UserF.create()

        project = ProjectF.create(
            add_contributors=[user],
            **{'status': 'inactive'}
        )

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 404)

    def test_get_public_project_with_contributor(self):
        user = UserF.create()

        project = ProjectF.create(
            add_contributors=[user],
            **{'isprivate': False}
        )

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, project.name)
        self.assertContains(response, '"can_contribute":true')

    def test_get_deleted_project_with_view_member(self):
        user = UserF.create()

        project = ProjectF.create(
            add_viewers=[user]
        )
        project.delete()

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 404)

    def test_get_private_project_with_view_member(self):
        user = UserF.create()

        project = ProjectF.create(
            add_viewers=[user]
        )

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, project.name)
        self.assertContains(response, '"can_contribute":false')

    def test_get_inactive_project_with_view_member(self):
        user = UserF.create()

        project = ProjectF.create(
            add_viewers=[user],
            **{'status': 'inactive'}
        )

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 404)

    def test_get_public_project_with_view_member(self):
        user = UserF.create()

        project = ProjectF.create(
            add_viewers=[user],
            **{'isprivate': False}
        )

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, project.name)
        self.assertContains(response, '"can_contribute":false')

    def test_get_deleted_project_with_non_member(self):
        user = UserF.create()

        project = ProjectF.create()
        project.delete()

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 404)

    def test_get_private_project_with_non_member(self):
        user = UserF.create()

        project = ProjectF.create()

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 404)

    def test_get_inactive_project_with_non_member(self):
        user = UserF.create()

        project = ProjectF.create(**{
            'status': 'inactive'
        })

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 404)

    def test_get_public_project_with_non_member(self):
        user = UserF.create()

        project = ProjectF.create(**{
            'isprivate': False
        })
        GroupingFactory(**{'project': project, 'isprivate': False})

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, project.name)
        self.assertContains(response, '"can_contribute":false')

    def test_get_public_project_with_anonymous(self):
        user = AnonymousUser()

        project = ProjectF.create(**{
            'isprivate': False
        })
        GroupingFactory(**{'project': project, 'isprivate': False})

        request = self.factory.get('/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, project.name)
        self.assertContains(response, '"can_contribute":false')


class ProjectContactAdminsTest(TestCase):
    def test_with_active_project(self):
        admin = UserF.create()
        email_user = UserF.create()

        project = ProjectF.create(
            add_admins=[admin],
            add_viewers=[email_user]
        )

        view = ProjectContactAdmins.as_view()
        url = reverse(
            'api:project_contact_admins',
            kwargs={
                'project_id': project.id
            }
        )
        request = APIRequestFactory().post(
            url,
            json.dumps({'email_text': 'Blah Blah'}),
            content_type='application/json'
        )
        force_authenticate(request, user=email_user)

        response = view(request, project_id=project.id).render()
        self.assertEqual(response.status_code, 204)
        self.assertEquals(len(mail.outbox), 2)

    def test_with_inactive_project(self):
        admin = UserF.create()
        email_user = UserF.create()

        project = ProjectF.create(
            add_admins=[admin],
            add_viewers=[email_user],
            **{'status': 'inactive'}
        )

        view = ProjectContactAdmins.as_view()
        url = reverse(
            'api:project_contact_admins',
            kwargs={
                'project_id': project.id
            }
        )
        request = APIRequestFactory().post(
            url,
            json.dumps({'email_text': 'Blah Blah'}),
            content_type='application/json'
        )
        force_authenticate(request, user=email_user)

        response = view(request, project_id=project.id).render()
        self.assertEqual(response.status_code, 404)
        self.assertEquals(len(mail.outbox), 0)

    def test_with_anonymous(self):
        admin = UserF.create()
        email_user = AnonymousUser()

        project = ProjectF.create(
            add_admins=[admin],
            **{'isprivate': False}
        )
        GroupingFactory.create(**{
            'isprivate': False,
            'project': project
        })

        view = ProjectContactAdmins.as_view()
        url = reverse(
            'api:project_contact_admins',
            kwargs={
                'project_id': project.id
            }
        )
        request = APIRequestFactory().post(
            url,
            json.dumps({'email_text': 'Blah Blah'}),
            content_type='application/json'
        )
        force_authenticate(request, user=email_user)

        response = view(request, project_id=project.id).render()
        self.assertEqual(response.status_code, 401)
        self.assertEquals(len(mail.outbox), 0)



from django import template
from django.core.urlresolvers import reverse

register = template.Library()


def get_view_group(grouping, group):
    disabled = ''
    message = ''
    if not grouping.project.isprivate and not grouping.isprivate:
        disabled = 'disabled="disabled"'
        message = '<p class="text-danger">This data grouping is public. To \
            restrict access, navigate to the <a href="%s">data grouping \
            settings</a> and revoke access from the public.</p>' % reverse(
                'admin:grouping_permissions',
                kwargs={
                    'project_id': grouping.project.id,
                    'grouping_id': grouping.id
                }
            )

    if group.viewgroups.filter(grouping=grouping).exists():
        return ('<li>'
                '%s<button type="button" name="%s" class="btn btn-default '
                'pull-right active grant-single" data-toggle="button" %s>'
                '<span class="text-danger">Revoke access</span></button>'
                '<strong>%s</strong><p>%s</p>'
                '</li>' % (
                    message,
                    grouping.id,
                    disabled,
                    grouping.name,
                    grouping.description)
                )
    else:
        return ('<li>'
                '%s<button type="button" name="%s" class="btn btn-default '
                'pull-right grant-single" data-toggle="button" %s><span '
                'class="text-success">Grant access</span></button>'
                '<strong>%s</strong><p>%s</p>'
                '</li>' % (
                    message,
                    grouping.id,
                    disabled,
                    grouping.name,
                    grouping.description)
                )


@register.simple_tag
def viewgroups(group):
    html = '<ul class="list-unstyled overview-list">'
    for grouping in group.project.groupings.all():
        html += get_view_group(grouping, group)

    html += '</ul>'

    return html



import datetime
import factory

from geokey.users.tests.model_factories import UserF
from geokey.projects.tests.model_factories import ProjectF
from geokey.categories.tests.model_factories import CategoryFactory

from ..models import Grouping, Rule


class GroupingFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Grouping

    name = factory.Sequence(lambda n: 'name_%d' % n)
    description = factory.LazyAttribute(lambda o: '%s description' % o.name)
    creator = factory.SubFactory(UserF)
    created_at = datetime.date(2014, 11, 11)
    status = 'active'
    isprivate = True
    project = factory.SubFactory(ProjectF)

    @factory.post_generation
    def add_viewers(self, create, extracted, **kwargs):
        if not create:
            return

        if extracted:
            from geokey.users.tests.model_factories import (
                UserGroupF, GroupingUserGroupFactory
            )
            group = UserGroupF(add_users=extracted, **{
                'project': self.project,
                'can_contribute': False
            })

            GroupingUserGroupFactory(**{
                'grouping': self,
                'usergroup': group
            })


class RuleFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Rule

    category = factory.SubFactory(CategoryFactory)
    grouping = factory.SubFactory(GroupingFactory)
    constraints = None
    status = 'active'



from django.test import TestCase
from django.contrib.auth.models import AnonymousUser
from django.core import mail

from nose.tools import raises

from geokey.datagroupings.tests.model_factories import GroupingFactory
from geokey.categories.tests.model_factories import CategoryFactory
from geokey.users.tests.model_factories import (
    UserF, UserGroupF, GroupingUserGroupFactory
)
from geokey.categories.models import Category

from .model_factories import ProjectF
from ..models import Project, Admins


class CreateProjectTest(TestCase):
    def test_create_project(self):
        creator = UserF.create()

        project = Project.create(
            'Test', 'Test desc', True, True, creator
        )
        self.assertIn(creator, project.admins.all())


class ProjectTest(TestCase):
    @raises(Project.DoesNotExist)
    def test_delete_project(self):
        project = ProjectF.create()
        project.delete()
        Project.objects.get(pk=project.id)

    def test_str(self):
        project = ProjectF.create(**{
            'name': 'Name',
            'status': 'inactive',
            'isprivate': False
        })
        self.assertEqual(
            str(project),
            'Name status: inactive private: False'
        )

    def test_get_role(self):
        admin = UserF.create()
        moderator = UserF.create()
        contributor = UserF.create()
        other = UserF.create()

        project = ProjectF.create(
            add_admins=[admin],
            add_moderators=[moderator],
            add_contributors=[contributor],
            add_viewers=[other]
        )

        self.assertEqual('administrator', project.get_role(admin))
        self.assertEqual('moderator', project.get_role(moderator))
        self.assertEqual('contributor', project.get_role(contributor))
        self.assertEqual('watcher', project.get_role(other))

    def test_re_order_categories(self):
        project = ProjectF.create()

        category_0 = CategoryFactory.create(**{'project': project})
        category_1 = CategoryFactory.create(**{'project': project})
        category_2 = CategoryFactory.create(**{'project': project})
        category_3 = CategoryFactory.create(**{'project': project})
        category_4 = CategoryFactory.create(**{'project': project})

        project.re_order_categories(
            [category_4.id, category_0.id, category_2.id,
             category_1.id,  category_3.id]
        )

        categories = project.categories.all()

        self.assertTrue(categories.ordered)
        self.assertEqual(categories[0], category_4)
        self.assertEqual(categories[1], category_0)
        self.assertEqual(categories[2], category_2)
        self.assertEqual(categories[3], category_1)
        self.assertEqual(categories[4], category_3)

    def test_re_order_categories_with_false_category(self):
        project = ProjectF.create()

        category_0 = CategoryFactory.create(**{'project': project})
        category_1 = CategoryFactory.create(**{'project': project})
        category_2 = CategoryFactory.create(**{'project': project})
        category_3 = CategoryFactory.create(**{'project': project})
        category_4 = CategoryFactory.create(**{'project': project})

        try:
            project.re_order_categories(
                [category_4.id, category_0.id, category_2.id,
                 category_1.id,  5854]
            )
        except Category.DoesNotExist:
            categories = project.categories.all()

            self.assertTrue(categories.ordered)
            self.assertEqual(categories[0].order, 0)
            self.assertEqual(categories[1].order, 1)
            self.assertEqual(categories[2].order, 2)
            self.assertEqual(categories[3].order, 3)
            self.assertEqual(categories[4].order, 4)


class PrivateProjectTest(TestCase):
    def setUp(self):
        self.moderator_view = UserF.create()
        self.moderator = UserF.create()
        self.contributor_view = UserF.create()
        self.contributor = UserF.create()
        self.viewer_view = UserF.create()
        self.viewer = UserF.create()
        self.some_dude = UserF.create()

        self.project = ProjectF.create(**{
            'isprivate': True,
            'everyone_contributes': 'false'
        })

        self.view = GroupingFactory(
            **{'project': self.project, 'isprivate': True}
        )
        self.moderators_view = UserGroupF(
            add_users=[self.moderator_view],
            **{
                'project': self.project,
                'can_moderate': True
            })
        self.contributors_view = UserGroupF(
            add_users=[self.contributor_view],
            **{
                'project': self.project,
                'can_contribute': True
            })
        self.viewers_view = UserGroupF(
            add_users=[self.viewer_view],
            **{
                'project': self.project,
                'can_contribute': False,
                'can_moderate': False
            })

        GroupingUserGroupFactory.create(
            **{'grouping': self.view, 'usergroup': self.moderators_view})
        GroupingUserGroupFactory.create(
            **{'grouping': self.view, 'usergroup': self.contributors_view})
        GroupingUserGroupFactory.create(
            **{'grouping': self.view, 'usergroup': self.viewers_view})

        self.moderators = UserGroupF(
            add_users=[self.moderator],
            **{
                'project': self.project,
                'can_moderate': True
            })
        self.contributors = UserGroupF(
            add_users=[self.contributor],
            **{
                'project': self.project,
                'can_contribute': True
            })
        self.viewers = UserGroupF(
            add_users=[self.viewer],
            **{
                'project': self.project,
                'can_contribute': False,
                'can_moderate': False
            })

    def test_is_admin(self):
        self.assertTrue(self.project.is_admin(self.project.creator))

        self.assertFalse(self.project.is_admin(self.moderator_view))
        self.assertFalse(self.project.is_admin(self.moderator))

        self.assertFalse(self.project.is_admin(self.contributor_view))
        self.assertFalse(self.project.is_admin(self.contributor))

        self.assertFalse(self.project.is_admin(self.viewer_view))
        self.assertFalse(self.project.is_admin(self.viewer))

        self.assertFalse(self.project.is_admin(self.some_dude))
        self.assertFalse(self.project.is_admin(AnonymousUser()))

    def test_can_access(self):
        self.assertTrue(self.project.can_access(self.project.creator))

        self.assertTrue(self.project.can_access(self.moderator_view))
        self.assertTrue(self.project.can_access(self.moderator))

        self.assertTrue(self.project.can_access(self.contributor_view))
        self.assertTrue(self.project.can_access(self.contributor))

        self.assertTrue(self.project.can_access(self.viewer_view))
        self.assertFalse(self.project.can_access(self.viewer))

        self.assertFalse(self.project.can_access(self.some_dude))
        self.assertFalse(self.project.can_access(AnonymousUser()))

    def can_contribute(self):
        self.assertTrue(self.project.can_contribute(
            self.project.creator))

        self.assertTrue(self.project.can_contribute(self.moderator_view))
        self.assertTrue(self.project.can_contribute(self.moderator))

        self.assertTrue(self.project.can_contribute(self.contributor_view))
        self.assertTrue(self.project.can_contribute(self.contributor))

        self.assertFalse(self.project.can_contribute(self.viewer_view))
        self.assertFalse(self.project.can_contribute(self.viewer))

        self.assertFalse(self.project.can_contribute(self.some_dude))
        self.assertFalse(self.project.can_contribute(AnonymousUser()))

    def can_moderate(self):
        self.assertTrue(self.project.can_moderate(self.project.creator))

        self.assertTrue(self.project.can_moderate(self.moderator_view))
        self.assertTrue(self.project.can_moderate(self.moderator))

        self.assertFalse(self.project.can_moderate(self.contributor_view))
        self.assertFalse(self.project.can_moderate(self.contributor))

        self.assertFalse(self.project.can_moderate(self.viewer_view))
        self.assertFalse(self.project.can_moderate(self.viewer))

        self.assertFalse(self.project.can_moderate(self.some_dude))
        self.assertFalse(self.project.can_moderate(AnonymousUser()))

    def is_involved(self):
        self.assertTrue(self.project.is_involved(self.project.creator))

        self.assertTrue(self.project.is_involved(self.moderator_view))
        self.assertTrue(self.project.is_involved(self.moderator))

        self.assertTrue(self.project.is_involved(self.contributor_view))
        self.assertTrue(self.project.is_involved(self.contributor))

        self.assertTrue(self.project.is_involved(self.viewer_view))
        self.assertTrue(self.project.is_involved(self.viewer))

        self.assertFalse(self.project.is_involved(self.some_dude))
        self.assertFalse(self.project.is_involved(AnonymousUser()))


class PrivateProjectTest_PublicView(TestCase):
    def setUp(self):
        self.moderator_view = UserF.create()
        self.moderator = UserF.create()
        self.contributor_view = UserF.create()
        self.contributor = UserF.create()
        self.viewer_view = UserF.create()
        self.viewer = UserF.create()
        self.some_dude = UserF.create()

        self.project = ProjectF.create(**{
            'isprivate': True,
            'everyone_contributes': 'false'
        })

        self.view = GroupingFactory(
            **{'project': self.project, 'isprivate': False}
        )
        self.moderators_view = UserGroupF(
            add_users=[self.moderator_view],
            **{
                'project': self.project,
                'can_moderate': True
            })
        self.contributors_view = UserGroupF(
            add_users=[self.contributor_view],
            **{
                'project': self.project,
                'can_contribute': True
            })
        self.viewers_view = UserGroupF(
            add_users=[self.viewer_view],
            **{
                'project': self.project,
                'can_contribute': False,
                'can_moderate': False
            })

        GroupingUserGroupFactory.create(
            **{'grouping': self.view, 'usergroup': self.moderators_view})
        GroupingUserGroupFactory.create(
            **{'grouping': self.view, 'usergroup': self.contributors_view})
        GroupingUserGroupFactory.create(
            **{'grouping': self.view, 'usergroup': self.viewers_view})

        self.moderators = UserGroupF(
            add_users=[self.moderator],
            **{
                'project': self.project,
                'can_moderate': True
            })
        self.contributors = UserGroupF(
            add_users=[self.contributor],
            **{
                'project': self.project,
                'can_contribute': True
            })
        self.viewers = UserGroupF(
            add_users=[self.viewer],
            **{
                'project': self.project,
                'can_contribute': False,
                'can_moderate': False
            })

    def test_is_admin(self):
        self.assertTrue(self.project.is_admin(self.project.creator))

        self.assertFalse(self.project.is_admin(self.moderator_view))
        self.assertFalse(self.project.is_admin(self.moderator))

        self.assertFalse(self.project.is_admin(self.contributor_view))
        self.assertFalse(self.project.is_admin(self.contributor))

        self.assertFalse(self.project.is_admin(self.viewer_view))
        self.assertFalse(self.project.is_admin(self.viewer))

        self.assertFalse(self.project.is_admin(self.some_dude))
        self.assertFalse(self.project.is_admin(AnonymousUser()))

    def test_can_access(self):
        self.assertTrue(self.project.can_access(self.project.creator))

        self.assertTrue(self.project.can_access(self.moderator_view))
        self.assertTrue(self.project.can_access(self.moderator))

        self.assertTrue(self.project.can_access(self.contributor_view))
        self.assertTrue(self.project.can_access(self.contributor))

        self.assertTrue(self.project.can_access(self.viewer_view))
        self.assertFalse(self.project.can_access(self.viewer))

        self.assertFalse(self.project.can_access(self.some_dude))
        self.assertFalse(self.project.can_access(AnonymousUser()))

    def can_contribute(self):
        self.assertTrue(self.project.can_contribute(self.project.creator))

        self.assertTrue(self.project.can_contribute(self.moderator_view))
        self.assertTrue(self.project.can_contribute(self.moderator))

        self.assertTrue(self.project.can_contribute(self.contributor_view))
        self.assertTrue(self.project.can_contribute(self.contributor))

        self.assertFalse(self.project.can_contribute(self.viewer_view))
        self.assertFalse(self.project.can_contribute(self.viewer))

        self.assertFalse(self.project.can_contribute(self.some_dude))
        self.assertFalse(self.project.can_contribute(AnonymousUser()))

    def can_moderate(self):
        self.assertTrue(self.project.can_moderate(self.project.creator))

        self.assertTrue(self.project.can_moderate(self.moderator_view))
        self.assertTrue(self.project.can_moderate(self.moderator))

        self.assertFalse(self.project.can_moderate(self.contributor_view))
        self.assertFalse(self.project.can_moderate(self.contributor))

        self.assertFalse(self.project.can_moderate(self.viewer_view))
        self.assertFalse(self.project.can_moderate(self.viewer))

        self.assertFalse(self.project.can_moderate(self.some_dude))
        self.assertFalse(self.project.can_moderate(AnonymousUser()))

    def is_involved(self):
        self.assertTrue(self.project.is_involved(self.project.creator))

        self.assertTrue(self.project.is_involved(self.moderator_view))
        self.assertTrue(self.project.is_involved(self.moderator))

        self.assertTrue(self.project.is_involved(self.contributor_view))
        self.assertTrue(self.project.is_involved(self.contributor))

        self.assertTrue(self.project.is_involved(self.viewer_view))
        self.assertTrue(self.project.is_involved(self.viewer))

        self.assertFalse(self.project.is_involved(self.some_dude))
        self.assertFalse(self.project.is_involved(AnonymousUser()))


class EveryoneContributesTest(TestCase):
    def test(self):
        project = ProjectF.create(**{
            'isprivate': False,
            'everyone_contributes': 'auth'
        })

        self.assertTrue(project.can_contribute(UserF.create()))
        self.assertFalse(project.can_contribute(AnonymousUser()))

        project = ProjectF.create(**{
            'isprivate': False,
            'everyone_contributes': 'true'
        })

        self.assertTrue(project.can_contribute(UserF.create()))
        self.assertTrue(project.can_contribute(AnonymousUser()))


class PublicProjectTest(TestCase):
    def setUp(self):
        self.moderator_view = UserF.create()
        self.moderator = UserF.create()
        self.contributor_view = UserF.create()
        self.contributor = UserF.create()
        self.viewer_view = UserF.create()
        self.viewer = UserF.create()
        self.some_dude = UserF.create()

        self.project = ProjectF.create(**{
            'isprivate': False,
            'everyone_contributes': 'false'
        })

        self.view = GroupingFactory(
            **{'project': self.project, 'isprivate': True}
        )
        self.moderators_view = UserGroupF(
            add_users=[self.moderator_view],
            **{
                'project': self.project,
                'can_moderate': True
            })
        self.contributors_view = UserGroupF(
            add_users=[self.contributor_view],
            **{
                'project': self.project,
                'can_contribute': True
            })
        self.viewers_view = UserGroupF(
            add_users=[self.viewer_view],
            **{
                'project': self.project,
                'can_contribute': False,
                'can_moderate': False
            })

        GroupingUserGroupFactory.create(
            **{'grouping': self.view, 'usergroup': self.moderators_view})
        GroupingUserGroupFactory.create(
            **{'grouping': self.view, 'usergroup': self.contributors_view})
        GroupingUserGroupFactory.create(
            **{'grouping': self.view, 'usergroup': self.viewers_view})

        self.moderators = UserGroupF(
            add_users=[self.moderator],
            **{
                'project': self.project,
                'can_moderate': True
            })
        self.contributors = UserGroupF(
            add_users=[self.contributor],
            **{
                'project': self.project,
                'can_contribute': True
            })
        self.viewers = UserGroupF(
            add_users=[self.viewer],
            **{
                'project': self.project,
                'can_contribute': False,
                'can_moderate': False
            })

    def test_is_admin(self):
        self.assertTrue(self.project.is_admin(self.project.creator))

        self.assertFalse(self.project.is_admin(self.moderator_view))
        self.assertFalse(self.project.is_admin(self.moderator))

        self.assertFalse(self.project.is_admin(self.contributor_view))
        self.assertFalse(self.project.is_admin(self.contributor))

        self.assertFalse(self.project.is_admin(self.viewer_view))
        self.assertFalse(self.project.is_admin(self.viewer))

        self.assertFalse(self.project.is_admin(self.some_dude))
        self.assertFalse(self.project.is_admin(AnonymousUser()))

    def test_can_access(self):
        self.assertTrue(self.project.can_access(self.project.creator))

        self.assertTrue(self.project.can_access(self.moderator_view))
        self.assertTrue(self.project.can_access(self.moderator))

        self.assertTrue(self.project.can_access(self.contributor_view))
        self.assertTrue(self.project.can_access(self.contributor))

        self.assertTrue(self.project.can_access(self.viewer_view))
        self.assertFalse(self.project.can_access(self.viewer))

        self.assertFalse(self.project.can_access(self.some_dude))
        self.assertFalse(self.project.can_access(AnonymousUser()))

    def can_contribute(self):
        self.assertTrue(self.project.can_contribute(self.project.creator))

        self.assertTrue(self.project.can_contribute(self.moderator_view))
        self.assertTrue(self.project.can_contribute(self.moderator))

        self.assertTrue(self.project.can_contribute(self.contributor_view))
        self.assertTrue(self.project.can_contribute(self.contributor))

        self.assertFalse(self.project.can_contribute(self.viewer_view))
        self.assertFalse(self.project.can_contribute(self.viewer))

        self.assertFalse(self.project.can_contribute(self.some_dude))
        self.assertFalse(self.project.can_contribute(AnonymousUser()))

    def can_moderate(self):
        self.assertTrue(self.project.can_moderate(self.project.creator))

        self.assertTrue(self.project.can_moderate(self.moderator_view))
        self.assertTrue(self.project.can_moderate(self.moderator))

        self.assertFalse(self.project.can_moderate(self.contributor_view))
        self.assertFalse(self.project.can_moderate(self.contributor))

        self.assertFalse(self.project.can_moderate(self.viewer_view))
        self.assertFalse(self.project.can_moderate(self.viewer))

        self.assertFalse(self.project.can_moderate(self.some_dude))
        self.assertFalse(self.project.can_moderate(AnonymousUser()))

    def is_involved(self):
        self.assertTrue(self.project.is_involved(self.project.creator))

        self.assertTrue(self.project.is_involved(self.moderator_view))
        self.assertTrue(self.project.is_involved(self.moderator))

        self.assertTrue(self.project.is_involved(self.contributor_view))
        self.assertTrue(self.project.is_involved(self.contributor))

        self.assertTrue(self.project.is_involved(self.viewer_view))
        self.assertTrue(self.project.is_involved(self.viewer))

        self.assertFalse(self.project.is_involved(self.some_dude))
        self.assertFalse(self.project.is_involved(AnonymousUser()))


class PublicProjectTest_PublicView(TestCase):
    def setUp(self):
        self.moderator_view = UserF.create()
        self.moderator = UserF.create()
        self.contributor_view = UserF.create()
        self.contributor = UserF.create()
        self.viewer_view = UserF.create()
        self.viewer = UserF.create()
        self.some_dude = UserF.create()

        self.project = ProjectF.create(**{
            'isprivate': False,
            'everyone_contributes': 'false'
        })

        self.view = GroupingFactory(
            **{'project': self.project, 'isprivate': False}
        )
        self.moderators_view = UserGroupF(
            add_users=[self.moderator_view],
            **{
                'project': self.project,
                'can_moderate': True
            })
        self.contributors_view = UserGroupF(
            add_users=[self.contributor_view],
            **{
                'project': self.project,
                'can_contribute': True
            })
        self.viewers_view = UserGroupF(
            add_users=[self.viewer_view],
            **{
                'project': self.project,
                'can_contribute': False,
                'can_moderate': False
            })

        GroupingUserGroupFactory.create(
            **{'grouping': self.view, 'usergroup': self.moderators_view})
        GroupingUserGroupFactory.create(
            **{'grouping': self.view, 'usergroup': self.contributors_view})
        GroupingUserGroupFactory.create(
            **{'grouping': self.view, 'usergroup': self.viewers_view})

        self.moderators = UserGroupF(
            add_users=[self.moderator],
            **{
                'project': self.project,
                'can_moderate': True
            })
        self.contributors = UserGroupF(
            add_users=[self.contributor],
            **{
                'project': self.project,
                'can_contribute': True
            })
        self.viewers = UserGroupF(
            add_users=[self.viewer],
            **{
                'project': self.project,
                'can_contribute': False,
                'can_moderate': False
            })

    def test_is_admin(self):
        self.assertTrue(self.project.is_admin(self.project.creator))

        self.assertFalse(self.project.is_admin(self.moderator_view))
        self.assertFalse(self.project.is_admin(self.moderator))

        self.assertFalse(self.project.is_admin(self.contributor_view))
        self.assertFalse(self.project.is_admin(self.contributor))

        self.assertFalse(self.project.is_admin(self.viewer_view))
        self.assertFalse(self.project.is_admin(self.viewer))

        self.assertFalse(self.project.is_admin(self.some_dude))
        self.assertFalse(self.project.is_admin(AnonymousUser()))

    def test_can_access(self):
        self.assertTrue(self.project.can_access(self.project.creator))

        self.assertTrue(self.project.can_access(self.moderator_view))
        self.assertTrue(self.project.can_access(self.moderator))

        self.assertTrue(self.project.can_access(self.contributor_view))
        self.assertTrue(self.project.can_access(self.contributor))

        self.assertTrue(self.project.can_access(self.viewer_view))
        self.assertTrue(self.project.can_access(self.viewer))

        self.assertTrue(self.project.can_access(self.some_dude))
        self.assertTrue(self.project.can_access(AnonymousUser()))

    def can_contribute(self):
        self.assertTrue(self.project.can_contribute(self.project.creator))

        self.assertTrue(self.project.can_contribute(self.moderator_view))
        self.assertTrue(self.project.can_contribute(self.moderator))

        self.assertTrue(self.project.can_contribute(self.contributor_view))
        self.assertTrue(self.project.can_contribute(self.contributor))

        self.assertFalse(self.project.can_contribute(self.viewer_view))
        self.assertFalse(self.project.can_contribute(self.viewer))

        self.assertFalse(self.project.can_contribute(self.some_dude))
        self.assertFalse(self.project.can_contribute(AnonymousUser()))

    def can_moderate(self):
        self.assertTrue(self.project.can_moderate(self.project.creator))

        self.assertTrue(self.project.can_moderate(self.moderator_view))
        self.assertTrue(self.project.can_moderate(self.moderator))

        self.assertFalse(self.project.can_moderate(self.contributor_view))
        self.assertFalse(self.project.can_moderate(self.contributor))

        self.assertFalse(self.project.can_moderate(self.viewer_view))
        self.assertFalse(self.project.can_moderate(self.viewer))

        self.assertFalse(self.project.can_moderate(self.some_dude))
        self.assertFalse(self.project.can_moderate(AnonymousUser()))

    def is_involved(self):
        self.assertTrue(self.project.is_involved(self.project.creator))

        self.assertTrue(self.project.is_involved(self.moderator_view))
        self.assertTrue(self.project.is_involved(self.moderator))

        self.assertTrue(self.project.is_involved(self.contributor_view))
        self.assertTrue(self.project.is_involved(self.contributor))

        self.assertTrue(self.project.is_involved(self.viewer_view))
        self.assertTrue(self.project.is_involved(self.viewer))

        self.assertFalse(self.project.is_involved(self.some_dude))
        self.assertFalse(self.project.is_involved(AnonymousUser()))


class ProjectContactAdminsTest(TestCase):
    def test_all_contacts(self):
        admin = UserF.create()
        contributor = UserF.create()
        email_user = UserF.create()

        project = ProjectF.create(
            add_admins=[admin],
            add_contributors=[contributor]
        )

        project.contact_admins(email_user, 'Test email')
        # Should be 2 because project creator is admin too
        self.assertEquals(len(mail.outbox), 2)

    def test_selected_contacts(self):
        admin = UserF.create()
        contributor = UserF.create()
        email_user = UserF.create()

        project = ProjectF.create(
            add_admins=[admin],
            add_contributors=[contributor]
        )

        admin_rel = Admins.objects.get(project=project, user=admin)
        admin_rel.contact = False
        admin_rel.save()

        project.contact_admins(email_user, 'Test email')
        self.assertEquals(len(mail.outbox), 1)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('datagroupings', '0002_auto_20150106_1338'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='grouping',
            options={'ordering': ['name']},
        ),
    ]



from django import forms
from django.utils.html import strip_tags

from .models import Grouping


class GroupingCreateForm(forms.ModelForm):
    """
    Validates the inputs against the model definition.
    Used in .views.ViewAdminCreateView
    """
    class Meta:
        model = Grouping
        fields = ('name', 'description', 'isprivate')

    def clean(self):
        """
        Cleans incoming data and returnes cleaned data. Removes HTML tags from
        name and description.

        Return
        ------
        dict
            Cleaned form data.
        """
        cleaned_data = super(GroupingCreateForm, self).clean()
        cleaned_data['name'] = strip_tags(cleaned_data['name'])
        cleaned_data['description'] = strip_tags(cleaned_data['description'])

        return cleaned_data



from geokey.core.serializers import FieldSelectorSerializer
from rest_framework.serializers import SerializerMethodField

from geokey.contributions.serializers import ContributionSerializer

from .models import Grouping


class GroupingSerializer(FieldSelectorSerializer):
    """
    Serializer for geokey.datagroupings.models.Grouping
    """
    contributions = SerializerMethodField('get_data')
    num_contributions = SerializerMethodField()

    class Meta:
        model = Grouping
        depth = 1
        fields = ('id', 'name', 'description', 'status',
                  'created_at', 'contributions', 'isprivate', 'status',
                  'num_contributions')
        read_only_fields = ('id', 'name', 'created_at')

    def get_data(self, obj):
        """
        Returns all serialized contributions accessable through the data
        grouping.

        Parameter
        ---------
        obj : geokey.datagroupings.models.Grouping
            Grouping that is serialised

        Returns
        -------
        list
            serialised contribution objects
        """
        user = self.context.get('user')

        serializer = ContributionSerializer(
            obj.data(user),
            many=True,
            context={'project': obj.project, 'user': user}
        )
        return serializer.data

    def get_num_contributions(self, obj):
        """
        Returns the number of contributions accessable through the view.

        Parameter
        ---------
        obj : geokey.datagroupings.models.Grouping
            Grouping that is serialised

        Returns
        -------
        int
            The number of contributions in the data grouping
        """
        user = self.context.get('user')
        return obj.data(user).count()
